@@ -1,10 +1,13 @@
-import cma
 import math
-import numpy as np
+
+import cma
 
 from linien.common import MHz, Vpp
-from linien.server.optimization.utils import get_max_slope, optimize_phase_from_iq, \
-    FINAL_ZOOM_FACTOR
+from linien.server.optimization.utils import (
+    FINAL_ZOOM_FACTOR,
+    get_max_slope,
+    optimize_phase_from_iq,
+)
 
 
 class NoOptimizationEngine:
@@ -43,19 +46,36 @@ class OneDimensionalOptimizationEngine:
 
 
 class MultiDimensionalOptimizationEngine:
-    def __init__(self, bounds):
+    def __init__(self, bounds, x0=None):
+        self.bounds = bounds
+
+        if x0 is not None:
+            x0_converted = self.params_to_internal(x0)
+        else:
+            x0_converted = [0.5 for v in bounds]
+
         self.es = cma.CMAEvolutionStrategy(
-            [np.mean(v) for v in bounds],
+            x0_converted,
             0.5,
-            {
-                'bounds': [[v[0] for v in bounds], [v[1] for v in bounds]]
-            }
+            {"bounds": [[0 for v in bounds], [1 for v in bounds]]},
         )
 
         self._pending = []
         self._done = []
         self._results = []
 
+    def params_to_internal(self, parameters):
+        new_parameters = []
+        for [min_, max_], param in zip(self.bounds, parameters):
+            new_parameters.append((param - min_) / (max_ - min_))
+        return new_parameters
+
+    def internal_to_params(self, internal):
+        parameters = []
+        for [min_, max_], param in zip(self.bounds, internal):
+            parameters.append((param * (max_ - min_)) + min_)
+        return parameters
+
     def finished(self):
         return self.es.stop()
 
@@ -63,7 +83,7 @@ class MultiDimensionalOptimizationEngine:
         if not self._pending:
             self._pending = self.es.ask()
 
-        return self._pending.pop()
+        return self.internal_to_params(self._pending.pop())
 
     def tell(self, fitness, parameters):
         self._results.append(fitness)
@@ -71,8 +91,7 @@ class MultiDimensionalOptimizationEngine:
 
         if not self._pending:
             self.es.tell(
-                self._done,
-                self._results
+                [self.params_to_internal(p) for p in self._done], self._results
             )
             self._results = []
             self._done = []
@@ -88,13 +107,13 @@ class OptimizerEngine:
         self.all_params = [
             params.modulation_frequency,
             params.modulation_amplitude,
-            self.get_demod_phase_param()
-        ]
-        self.params_before_start = [
-            p.value for p in self.all_params
+            self.get_demod_phase_param(),
         ]
+        self.params_before_start = [p.value for p in self.all_params]
 
-        self.parameters.optimization_optimized_parameters.value = self.params_before_start
+        self.parameters.optimization_optimized_parameters.value = (
+            self.params_before_start
+        )
 
         self.initial_slope = None
         self.last_parameters = None
@@ -108,24 +127,32 @@ class OptimizerEngine:
 
         if params.optimization_mod_freq_enabled.value:
             self.to_optimize.append(params.modulation_frequency)
-            freqs = list(sorted([
-                params.optimization_mod_freq_min.value * MHz,
-                params.optimization_mod_freq_max.value * MHz
-            ]))
+            freqs = list(
+                sorted(
+                    [
+                        params.optimization_mod_freq_min.value * MHz,
+                        params.optimization_mod_freq_max.value * MHz,
+                    ]
+                )
+            )
             self.bounds.append(freqs)
 
         if params.optimization_mod_amp_enabled.value:
             self.to_optimize.append(params.modulation_amplitude)
-            ampls = list(sorted([
-                params.optimization_mod_amp_min.value * Vpp,
-                params.optimization_mod_amp_max.value * Vpp
-            ]))
+            ampls = list(
+                sorted(
+                    [
+                        params.optimization_mod_amp_min.value * Vpp,
+                        params.optimization_mod_amp_max.value * Vpp,
+                    ]
+                )
+            )
             self.bounds.append(ampls)
 
         self.opt = [
             NoOptimizationEngine,
             OneDimensionalOptimizationEngine,
-            MultiDimensionalOptimizationEngine
+            MultiDimensionalOptimizationEngine,
         ][len(self.bounds)]([[0, 1]] * len(self.bounds))
 
     def request_and_set_new_parameters(self, use_initial_parameters=False):
@@ -147,7 +174,7 @@ class OptimizerEngine:
             self.last_parameters = list(new_params_converted)
             self.last_parameters_internal = list(new_params)
 
-        self.control.exposed_write_data()
+        self.control.exposed_write_registers()
         self.control.continue_acquisition()
 
     def finished(self):
@@ -160,16 +187,12 @@ class OptimizerEngine:
 
         return (
             self.parameters.demodulation_phase_a,
-            self.parameters.demodulation_phase_b
-        )[
-            0 if not dual_channel else (0, 1)[channel]
-        ]
+            self.parameters.demodulation_phase_b,
+        )[0 if not dual_channel else (0, 1)[channel]]
 
     def tell(self, i, q):
         if self.initial_slope is None:
-            self.initial_slope = get_max_slope(
-                i, FINAL_ZOOM_FACTOR
-            )
+            self.initial_slope = get_max_slope(i, FINAL_ZOOM_FACTOR)
 
         optimized_phase, optimized_slope = optimize_phase_from_iq(
             i, q, FINAL_ZOOM_FACTOR
@@ -192,13 +215,13 @@ class OptimizerEngine:
                 for param, value in zip(self.to_optimize, self.last_parameters):
                     idx = {
                         self.parameters.modulation_frequency: 0,
-                        self.parameters.modulation_amplitude: 1
+                        self.parameters.modulation_amplitude: 1,
                     }[param]
                     complete_parameter_set[idx] = value
 
             params.optimization_optimized_parameters.value = complete_parameter_set
 
-        print('improvement %d' % (improvement * 100))
+        print("improvement %d" % (improvement * 100))
 
         fitness = math.log(1 / optimized_slope)
 
@@ -209,4 +232,4 @@ class OptimizerEngine:
         optimized = self.parameters.optimization_optimized_parameters.value
 
         for param, value in zip(self.all_params, optimized):
-            param.value = value
\ No newline at end of file
+            param.value = value
