@@ -1,30 +1,29 @@
 import os
 import sys
 
-sys.path += ["../../"]
-import rpyc
-import click
+# add the two parent directories to the path so linien and autolock can be found
+dir_name = os.path.dirname(os.path.abspath(__file__))
+sys.path += [os.path.join(dir_name, "..", "..")]
+sys.path += [os.path.join(dir_name, "..")]
+
 import _thread
 import pickle
-import numpy as np
-
-from rpyc.utils.server import ThreadedServer
-from rpyc.utils.authenticators import AuthenticationError
+import threading
 from random import random
+from time import sleep
 
+import click
+import numpy as np
+import rpyc
 from autolock.autolock import Autolock
 from parameters import Parameters
+from rpyc.utils.authenticators import AuthenticationError
+from rpyc.utils.server import ThreadedServer
 
+from linien.common import N_POINTS, check_plot_data, pack, unpack, update_signal_history
 from linien.config import DEFAULT_SERVER_PORT
-from linien.common import (
-    check_plot_data,
-    update_control_signal_history,
-    N_POINTS,
-    pack,
-    unpack,
-)
-from linien.server.parameter_store import ParameterStore
 from linien.server.optimization.optimization import OptimizeSpectroscopy
+from linien.server.parameter_store import ParameterStore
 from linien.server.pid_optimization.pid_optimization import (
     PIDOptimization,
     PSDAcquisition,
@@ -54,7 +53,7 @@ class BaseService(rpyc.Service):
         self.parameters._get_param(param_name).value = unpack(value)
 
     def exposed_init_parameter_sync(self, uuid):
-        return self.parameters.init_parameter_sync(uuid)
+        return pack(list(self.parameters.init_parameter_sync(uuid)))
 
     def exposed_register_remote_listener(self, uuid, param_name):
         return self.parameters.register_remote_listener(uuid, param_name)
@@ -105,14 +104,18 @@ class RedPitayaControlService(BaseService):
                         return
 
                     self.parameters.to_plot.value = plot_data
+                    self._generate_signal_stats(data_loaded)
 
-                    self.parameters.control_signal_history.value = (
-                        update_control_signal_history(
-                            self.parameters.control_signal_history.value,
-                            data_loaded,
-                            is_locked,
-                            self.parameters.control_signal_history_length.value,
-                        )
+                    # update signal history (if in locked state)
+                    (
+                        self.parameters.control_signal_history.value,
+                        self.parameters.monitor_signal_history.value,
+                    ) = update_signal_history(
+                        self.parameters.control_signal_history.value,
+                        self.parameters.monitor_signal_history.value,
+                        data_loaded,
+                        is_locked,
+                        self.parameters.control_signal_history_length.value,
                     )
                 else:
                     self.parameters.acquisition_raw_data.value = plot_data
@@ -121,7 +124,35 @@ class RedPitayaControlService(BaseService):
         self.pause_acquisition()
         self.continue_acquisition()
 
-    def exposed_write_data(self):
+    def run_periodic_timer(self):
+        """Starts a timer that increases the `ping` parameter once per second.
+        Its purpose is to allow for periodic tasks on the server:
+        just register an `on_change` listener for this parameter.
+        """
+
+        def send_ping():
+            while True:
+                self.parameters.ping.value = self.parameters.ping.value + 1
+                print("ping", self.parameters.ping.value)
+                sleep(1)
+
+        t = threading.Thread(target=send_ping)
+        t.daemon = True
+        t.start()
+        self._periodic_timer = t
+
+    def _generate_signal_stats(self, to_plot):
+        stats = {}
+
+        for signal_name, signal in to_plot.items():
+            stats["%s_mean" % signal_name] = np.mean(signal)
+            stats["%s_std" % signal_name] = np.std(signal)
+            stats["%s_max" % signal_name] = np.max(signal)
+            stats["%s_min" % signal_name] = np.min(signal)
+
+        self.parameters.signal_stats.value = stats
+
+    def exposed_write_registers(self):
         """Syncs the parameters with the FPGA registers."""
         self.registers.write_registers()
 
@@ -135,7 +166,8 @@ class RedPitayaControlService(BaseService):
 
     def exposed_start_autolock(self, x0, x1, spectrum, additional_spectra=None):
         spectrum = pickle.loads(spectrum)
-        start_watching = self.parameters.watch_lock.value
+        # start_watching = self.parameters.watch_lock.value
+        start_watching = False
         auto_offset = self.parameters.autolock_determine_offset.value
 
         if not self.task_running():
@@ -168,12 +200,12 @@ class RedPitayaControlService(BaseService):
             self.parameters.task.value = PIDOptimization(self, self.parameters)
             self.parameters.task.value.run()
 
-    def exposed_start_ramp(self):
+    def exposed_start_sweep(self):
         self.pause_acquisition()
 
         self.parameters.combined_offset.value = 0
         self.parameters.lock.value = False
-        self.exposed_write_data()
+        self.exposed_write_registers()
 
         self.continue_acquisition()
 
@@ -181,7 +213,7 @@ class RedPitayaControlService(BaseService):
         self.pause_acquisition()
 
         self.parameters.lock.value = True
-        self.exposed_write_data()
+        self.exposed_write_registers()
 
         self.continue_acquisition()
 
@@ -234,13 +266,13 @@ class FakeRedPitayaControl(BaseService):
         super().__init__()
         self.exposed_is_locked = None
 
-    def exposed_write_data(self):
+    def exposed_write_registers(self):
         pass
 
     def run_acquiry_loop(self):
         import threading
-        from time import sleep
         from random import randint
+        from time import sleep
 
         def run():
             while True:
@@ -260,6 +292,9 @@ class FakeRedPitayaControl(BaseService):
         t.daemon = True
         t.start()
 
+    def run_periodic_timer(self):
+        pass
+
     def exposed_shutdown(self):
         _thread.interrupt_main()
         os._exit(0)
@@ -321,7 +356,8 @@ def run_server(port, fake=False, remote_rp=False):
             control = RedPitayaControlService()
 
     control.run_acquiry_loop()
-    control.exposed_write_data()
+    control.run_periodic_timer()
+    control.exposed_write_registers()
 
     failed_auth_counter = {"c": 0}
 
@@ -356,7 +392,10 @@ def run_server(port, fake=False, remote_rp=False):
         return sock, None
 
     t = ThreadedServer(
-        control, port=port, authenticator=username_and_password_authenticator
+        control,
+        port=port,
+        authenticator=username_and_password_authenticator,
+        protocol_config={"allow_pickle": True},
     )
     t.start()
 
