@@ -10,7 +10,7 @@ setuptools.setup(
     author="Benjamin Wiegand",
     author_email="highwaychile@posteo.de",
     description="Server of linien that runs on RedPitaya.",
-    long_description="Have a look at the 'linien' package for installation instructions.",
+    long_description="Have a look at the 'linien' package for installation instructions.",  # noqa: E501
     long_description_content_type="text/x-rst",
     url="https://hermitdemschoenenleben.github.io/linien/",
     packages=[
@@ -36,20 +36,20 @@ setuptools.setup(
         "scipy>=0.17.0",
     ],
     scripts=[
-        "linien/server/linien_start_server",
-        "linien/server/linien_stop_server",
-        "linien/server/linien_install_requirements",
-        "linien/server/linien_start_ethernet_blinking",
-        "linien/server/linien_stop_ethernet_blinking",
+        "linien/server/linien_start_server.sh",
+        "linien/server/linien_stop_server.sh",
+        "linien/server/linien_install_requirements.sh",
+        "linien/server/linien_start_ethernet_blinking.sh",
+        "linien/server/linien_stop_ethernet_blinking.sh",
     ],
     package_data={
         "": [
             "linien.bin",
-            "linien_start_server",
-            "linien_stop_server",
-            "linien_start_ethernet_blinking",
-            "linien_stop_ethernet_blinking",
-            "linien_install_requirements",
+            "linien_start_server.sh",
+            "linien_stop_server.sh",
+            "linien_start_ethernet_blinking.sh",
+            "linien_stop_ethernet_blinking.sh",
+            "linien_install_requirements.sh",
             "VERSION",
         ]
     },
